#ifdef AUTHLDAP

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include "radiusd.h"


/*
 * BH 2001062901
 * Read Radius-LDAP Mappings
 * Thx to all free-radius developers for code
 */


#define MAX_LINE_LEN 160
#define GENERIC_ATTRIBUTE_ID "$GENERIC$"

extern struct LDAP_s *ldaps;


int read_mappings(char *filename)
{
	FILE	*mapfile;

	/* all buffers are of MAX_LINE_LEN so we can use sscanf without being afraid of buffer overflows */
	char	buf[MAX_LINE_LEN],
			itemType[MAX_LINE_LEN],
			radiusAttribute[MAX_LINE_LEN],
			ldapAttribute[MAX_LINE_LEN],
			*ptr;

	int	linenumber,
			token_count;

	TLDAP_RADIUS
			*pair;

	/* open the mappings file for reading */

	DEBUG("rlm_ldap: reading ldap<->radius mappings from file %s", filename);
	mapfile = fopen(filename, "r");

	if (mapfile == NULL) {
		log(L_ERR, "rlm_ldap: Opening file %s failed", filename);
		return -1; /* error */
	}

	/* read file line by line. Note that if line length exceed MAX_LINE_LEN, line numbers will be mixed up */

	linenumber = 0;

	while (fgets(buf, sizeof buf, mapfile)!=NULL) {

		linenumber++;

		/* strip comments */
		ptr = strchr(buf, '#');
		if (ptr) *ptr = 0;
		
		/* empty line */
		if (buf[0] == 0) continue;
		
		/* extract tokens from the string */		
		token_count = sscanf(buf, "%s %s %s", itemType, radiusAttribute, ldapAttribute);

		if (token_count <= 0) /* no tokens */			
			continue;

		if (token_count != 3) {
			log(L_ERR, "rlm_ldap: Skipping %s line %i: %s", filename, linenumber, buf);
			log(L_ERR, "rlm_ldap: Expected 3 tokens "
			       "(Item type, RADIUS Attribute and LDAP Attribute) but found only %i", token_count);
			continue;
		}

		/* create new TLDAP_RADIUS list node */
		pair = malloc(sizeof(TLDAP_RADIUS));

		pair->attr = strdup(ldapAttribute);
		pair->radius_attr = strdup(radiusAttribute);

		if ( (pair->attr == NULL) || (pair->radius_attr == NULL) ) {
			log(L_ERR, "rlm_ldap: Out of memory");
			fclose(mapfile);
			return -1;
		}
			
		/* push node to correct list */
		if (strcasecmp(itemType, "checkItem") == 0) {
			pair->next = ldaps->check_item_map;
			ldaps->check_item_map = pair;
		} else if (strcasecmp(itemType, "replyItem") == 0) {
			pair->next = ldaps->reply_item_map;
			ldaps->reply_item_map = pair;
		} else {
			log(L_ERR, "rlm_ldap: file %s: skipping line %i: unknown itemType %s", 
			       filename, linenumber, itemType);
			free(pair->attr);
			free(pair->radius_attr);
			free(pair);
			continue;
		}

		DEBUG("rlm_ldap: LDAP %s mapped to RADIUS %s",
		      pair->attr, pair->radius_attr);
	}
	
	fclose(mapfile);

	return 0; /* success */
}

#endif